#include "settings.h"
#include <iostream.h>

#define CONF_ADDPROP(_type, _name) \
void CPULoadSettings::Set##_name(_type value)\
{\
	_conf##_name = value;\
}\
\
_type CPULoadSettings::##_name()\
{\
	return _conf##_name;\
}\


CONF_ADDPROP(bigtime_t, Delay)
CONF_ADDPROP(rgb_color, GraphColor)
CONF_ADDPROP(int32, GraphType)
CONF_ADDPROP(int32, ShowScale)

//CONF_ADDPROP(BPath, AppPath)
CONF_ADDPROP(entry_ref, AppPath)
#undef CONF_ADDPROP

CPULoadSettings::CPULoadSettings()
{
	_confDelay = DEFAULT_DELAY;
	_confGraphColor = ui_color(B_MENU_ITEM_TEXT_COLOR);
	_confShowScale = Scale_Under;
	_confGraphType = Graph_Filled;

	BPath prefPath;

	find_directory(B_USER_SETTINGS_DIRECTORY, &prefPath);
	prefPath.Append(SETTINGS_FILE);
		
	_settingsFile.SetTo(prefPath.Path(), B_READ_WRITE | B_CREATE_FILE);
	
	if (_settingsMessage.Unflatten(&_settingsFile) == B_OK){
	
		if (_settingsMessage.FindInt32(CL_COLOR, (int32 *)&_confGraphColor) != B_OK)
			printf("CPULoadSettings::CPULoadSettings();\tFailed to load active boolean from settings file. Using default\n");
			
		if (_settingsMessage.FindInt64(CL_DELAY, &_confDelay) != B_OK)
			printf("CPULoadSettings::CPULoadSettings();\tFailed to load delay from settings file. Using default\n");

		if (_settingsMessage.FindInt32(CL_GRAPH_TYPE, &_confGraphType) != B_OK)
			printf("CPULoadSettings::CPULoadSettings();\tFailed to load graph type from settings file. Using default\n");

		if (_settingsMessage.FindInt32(CL_SHOW_SCALE, &_confShowScale) != B_OK)
			printf("CPULoadSettings::CPULoadSettings();\tFailed to load show scale from settings file. Using default\n");

//		if (_settingsMessage.FindFlat(CL_APP_PATH, (BFlattenable *) &_appPath) != B_OK)
//			printf("CPULoadSettings::CPULoadSettings();\tFailed to load application path.\n");

		if (_settingsMessage.FindRef(APP_PATH, &_confAppPath) != B_OK)
			printf("CPULoadSettings::CPULoadSettings();\tFailed to load application path.\n");
	}
	else
	{
		printf("CPULoadSettings::CPULoadSettings()\nUnable to open settings file (either corrupted or doesn't exist), using defaults.\n");		
	}
	
	_settingsFile.Unset();		
}

CPULoadSettings::~CPULoadSettings()
{
	BPath prefPath;
	find_directory(B_USER_SETTINGS_DIRECTORY, &prefPath);
	prefPath.Append(SETTINGS_FILE);
	
	//clobber existing settings and write in new ones
	_settingsFile.SetTo(prefPath.Path(), B_READ_WRITE | B_ERASE_FILE);


	//empty message and refill it with whatever has been set
	_settingsMessage.MakeEmpty();

	_settingsMessage.AddInt32(CL_COLOR, *(int32 *)(&_confGraphColor));
	_settingsMessage.AddInt64(CL_DELAY, _confDelay);
	_settingsMessage.AddInt32(CL_GRAPH_TYPE, _confGraphType);
	_settingsMessage.AddInt32(CL_SHOW_SCALE, _confShowScale);
//	_settingsMessage.AddFlat(CL_APP_PATH, &_appPath);
	_settingsMessage.AddRef(APP_PATH, &_confAppPath);

	//write message to settings file
	if (_settingsMessage.Flatten(&_settingsFile) != B_OK)
		printf("Error occurred writing settings\n");

	_settingsFile.Unset();	
}

